from sklearn.preprocessing import MinMaxScaler
import copy

def scale_input_data(input_data, metadata, ignore_integrated_scaler=False):
    """
        Scale input data from real values to normalized.

        Parameters
        ----------
        data : numpy.array
            Data as numpy array
        metadata : dict
            Model metadata
        ignore_integrated_scaler: bool
            If False(default), only scale data if model does not contain integrated scaler

        Returns
        -------
        numpy.array
            Scaled data
        """
    if ignore_integrated_scaler or not metadata['is_scaler_integrated']:
        return __scale_data_from_config(
            input_data,
            metadata['scaler_config']['input'],
            metadata['scaler_name']
        )
    else:
        return input_data



def scale_output_data(output_data, metadata, ignore_integrated_scaler=False):
    """
        Scale data from normalized values to real values. Use after prediction.

        Parameters
        ----------
        data : numpy.array
            Data as numpy array
        metadata : dict
            Model metadata
        ignore_integrated_scaler: bool
            If False(default), only scale data if model does not contain integrated scaler

        Returns
        -------
        numpy.array
            Scaled data
        """
    if ignore_integrated_scaler or not metadata['is_scaler_integrated']:
        recurrent_output_count = metadata.get('recurrent_output_count', None)
        config = metadata['scaler_config']['output']
        if recurrent_output_count is not None:
            config = copy.deepcopy(config)
            config['fit'][0] = config['fit'][0] * recurrent_output_count
            config['fit'][1] = config['fit'][1] * recurrent_output_count
        return __scale_data_from_config(
            output_data,
            config,
            metadata['scaler_name']
        )
    else:
        return output_data


def __scale_data_from_config(data, config, scaler_name='MinMaxScaler'):
    if scaler_name == 'MinMaxScaler':
        return __min_max_scale_data(
            data,
            config['fit'],
            config['feature_range'],
            config['inverse_transform']
        )
    else:
        raise Exception('Unknown scaler ' + scaler_name)


def __min_max_scale_data(data, fit, feature_range, inverse=False):
    scaler = MinMaxScaler(feature_range=feature_range)
    scaler.fit(fit)
    if inverse is True:
        return scaler.inverse_transform(data)
    else:
        return scaler.transform(data)



from .input_extras import remove_not_used_inputs
from .scaler_extras import scale_input_data, scale_output_data
from .series_extras import pack_input_data_for_series
from .series_extras import pack_output_data_for_series
from .rnn_extras import pack_input_data_for_rnn
from .rnn_extras import pack_output_data_for_rnn


def prepare_input_data(input_data, metadata):
    """
        Prepare the input for prediction with the following steps
            1. removing insignificant columns
            2. packing data for series
            3. scaling (normalizing) values
            4. pack data for recurrent neural network
        Parameters
        ----------
        input_data : numpy.array
            Input data as numpy array
        metadata : dict
            Model metadata

        Returns
        -------
        numpy.array
            Prepared values
        """
    used_inputs = remove_not_used_inputs(input_data, metadata)
    if metadata['has_rolling']:
        used_inputs = pack_input_data_for_series(used_inputs, metadata)
    used_inputs = scale_input_data(used_inputs, metadata)
    if metadata.get('recurrent_input_count', None) is not None and metadata.get('recurrent_output_count', None) is not None:
        used_inputs = pack_input_data_for_rnn(used_inputs, metadata)
    return used_inputs


def prepare_output_data(output_data, metadata):
    """
        Prepare the output for prediction with the following steps
            1. packing data for series or pack data for recurrent neural network
        Parameters
        ----------
        output_data : numpy.array
            Output data as numpy array
        metadata : dict
            Model metadata

        Returns
        -------
        numpy.array
            Prepared values
        """
    if metadata['has_rolling']:
        output_data = pack_output_data_for_series(output_data, metadata)
    if metadata.get('recurrent_input_count', None) is not None and metadata.get('recurrent_output_count', None) is not None:
        output_data = pack_output_data_for_rnn(output_data, metadata)
    return output_data